import rox
from rox import g, OptionsBox
from rcfile import Opt, OptionGroupRC, update_options

app_name = 'OroboROX'
keybutt_size = g.SizeGroup(g.SIZE_GROUP_HORIZONTAL)

def build_keybutt(wid, node, label, option):
    global keybutt_size
    b = g.Button()
    clr = rox.ButtonMixed(g.STOCK_CLEAR,_('Clear'))
    l = g.Label(label)
    l.set_alignment(1,0.5)
    l.set_padding(6,0)
    need_mod = node.getAttribute('need_mod')
    
    def clr_clicked(self):
        b.set_label('none') #This should not be translated!
        wid.check_widget(option)
        
    def keycut_handler(self, kev):
        m = kev.state
        k = kev.keyval
        
        if k == g.keysyms.Escape and not m:
            self.destroy()
            return
                    
        keys = ''
        label = ''

        if need_mod and not m: return
        if not k: return
                # TODO: if k is a modifier keyval, return
                # How do I check for that?
        
        if m & g.gdk.CONTROL_MASK:	keys += 'Control+' ; label +=_('Control')+'+'
        if m & g.gdk.SHIFT_MASK:	keys += 'Shift+' ; label +=_('Shift')+'+'
        if m & g.gdk.MOD1_MASK:		keys += 'Mod1+' ; label +=_('Mod1')+'+'
#		if m & g.gdk.MOD2_MASK:		keys += 'Mod2+' ; label +=_('Mod2')+'+' # ignore numlock
        if m & g.gdk.MOD3_MASK:		keys += 'Mod3+' ; label +=_('Mod3')+'+'
        if m & g.gdk.MOD4_MASK:		keys += 'Mod4+' ; label +=_('Mod4')+'+'
        if m & g.gdk.MOD5_MASK:		keys += 'Mod5+' ; label +=_('Mod5')+'+'

        if need_mod != '2': keys += str(g.gdk.keyval_name(k)) ; label += str(g.gdk.keyval_name(k))
        else: keys += 'none' ; label +=_('none')
        b.set_label(keys)
        #b.set_label(label)
        wid.check_widget(option)
        self.destroy()
        
    def get_keycut(self):
        w = rox.Window()
        w.set_position(g.WIN_POS_CENTER)
        w.set_title(_('Set shortcut'))
        l = g.Label(_("Press your desired keyboard shortcut\nfor '%s'\nor Escape to cancel...") % label)

        def grab(self,dummy):
            g.gdk.keyboard_grab(self.window,1)
            g.gdk.pointer_grab(self.window,0,0,self.window)
            
    #	def ungrab(self):
    #		g.gdk.keyboard_ungrab()
    #		g.gdk.pointer_ungrab()

        w.connect('key-release-event',keycut_handler)
        w.connect('map-event',grab)
    #	w.connect('destroy',ungrab)
        w.add(l)
        w.set_border_width(30)
        w.show_all()
    
    def get():
        return b.get_label()
    def set():
        if option.value:
            b.set_label(option.value)
        else:
            b.set_label('')
    wid.handlers[option] = (get, set)

    box = g.HBox(False, 4)
    b.connect('clicked',get_keycut)
    #b.set_size_request(180,-1)
    keybutt_size.add_widget(b)
    wid.tips.set_tip(b,_('Click to grab a new keyboard shortcut'))
    wid.tips.set_tip(clr,_('Click to clear the keyboard shortcut'))
    clr.connect('clicked',clr_clicked)
    box.pack_start(l, True, True)
    box.pack_end(clr, False, False)
    box.pack_end(b, False, False)

    return [box]
    
OptionsBox.widget_registry['keybutt'] = build_keybutt

rox.app_options = OptionGroupRC(app_name,'keyboardrc')

Opt('hide_window_key')
Opt('close_window_key')
Opt('toggle_fullscreen_window_key')
Opt('maximize_window_key')
Opt('shade_window_key')
Opt('cycle_windows_key')
Opt('raise_window_layer_key')
Opt('lower_window_layer_key')
Opt('move_window_up_key')
Opt('move_window_down_key')
Opt('move_window_left_key')
Opt('move_window_right_key')
Opt('next_workspace_key')
Opt('prev_workspace_key')
Opt('add_workspace_key')
Opt('del_workspace_key')
Opt('stick_window_key')
Opt('workspace_1_key')
Opt('workspace_2_key')
Opt('workspace_3_key')
Opt('workspace_4_key')
Opt('workspace_5_key')
Opt('workspace_6_key')
Opt('workspace_7_key')
Opt('workspace_8_key')
Opt('workspace_9_key')
Opt('move_window_next_workspace_key')
Opt('move_window_prev_workspace_key')
Opt('move_window_workspace_1_key')
Opt('move_window_workspace_2_key')
Opt('move_window_workspace_3_key')
Opt('move_window_workspace_4_key')
Opt('move_window_workspace_5_key')
Opt('move_window_workspace_6_key')
Opt('move_window_workspace_7_key')
Opt('move_window_workspace_8_key')
Opt('move_window_workspace_9_key')
Opt('maximize_vert_key')
Opt('maximize_horiz_key')
Opt('resize_window_up_key')
Opt('resize_window_down_key')
Opt('resize_window_left_key')
Opt('resize_window_right_key')
Opt('move_top_left_key')
Opt('move_top_right_key')
Opt('move_bottom_left_key')
Opt('move_bottom_right_key')
Opt('window_ops_modifiers')
Opt('toggle_fullscreen_window_all_key')
Opt('maximize_window_all_key')
Opt('maximize_vert_all_key')
Opt('maximize_horiz_all_key')
Opt('toggle_borders_key')

rox.app_options.add_notify(update_options)
rox.app_options.notify()
rox.edit_options()
