import rox
from rox import g, OptionsBox
from rox.options import Option
from rcfile import Opt, OptionGroupRC, update_options

### TODO:
###
### A special widget for button layout? (drag and drop of symbols)
###

app_name = 'OroboROX'

keybutt_size = g.SizeGroup(g.SIZE_GROUP_HORIZONTAL)

def build_keybutt(wid, node, label, option):
	global keybutt_size
	b = g.Button()
	clr = rox.ButtonMixed(g.STOCK_CLEAR,_('Clear'))
	l = g.Label(label)
	l.set_alignment(1,0.5)
	l.set_padding(6,0)
        need_mod = node.getAttribute('need_mod')
	
	def clr_clicked(self):
		b.set_label('none') #This should not be translated!
		wid.check_widget(option)
		
	def keycut_handler(self, kev):
                m = kev.state
                k = kev.keyval
		
                if k == g.keysyms.Escape and not m:
                    self.destroy()
                    return
                    
		keys = ''

		if need_mod and not m: return
		if not k: return
                # TODO: if k is a modifier keyval, return
                # How do I check for that?
		
		if m & g.gdk.CONTROL_MASK:	keys += 'Control+'
		if m & g.gdk.SHIFT_MASK:	keys += 'Shift+'
		if m & g.gdk.MOD1_MASK:		keys += 'Mod1+'
#		if m & g.gdk.MOD2_MASK:		keys += 'Mod2+' # ignore numlock
		if m & g.gdk.MOD3_MASK:		keys += 'Mod3+'
		if m & g.gdk.MOD4_MASK:		keys += 'Mod4+'
		if m & g.gdk.MOD5_MASK:		keys += 'Mod5+'

		if need_mod != '2': keys += str(g.gdk.keyval_name(k))
                else: keys += 'none'
		b.set_label(keys)
		wid.check_widget(option)
		self.destroy()
		
	def get_keycut(self):
		w = rox.Window()
		w.set_position(g.WIN_POS_CENTER)
		w.set_title(_('Set shortcut'))
		l = g.Label(_("Press your desired keyboard shortcut\nfor '%s'\nor Escape to cancel...") % label)

		def grab(self,dummy):
			g.gdk.keyboard_grab(self.window,1)
			g.gdk.pointer_grab(self.window,0,0,self.window)
			
	#	def ungrab(self):
	#		g.gdk.keyboard_ungrab()
	#		g.gdk.pointer_ungrab()

		w.connect('key-release-event',keycut_handler)
		w.connect('map-event',grab)
	#	w.connect('destroy',ungrab)
		w.add(l)
		w.set_border_width(30)
		w.show_all()
	
	def get():
		return b.get_label()
	def set():
		if option.value:
			b.set_label(option.value)
		else:
			b.set_label('')
	wid.handlers[option] = (get, set)

	box = g.HBox(False, 4)
	b.connect('clicked',get_keycut)
	#b.set_size_request(180,-1)
	keybutt_size.add_widget(b)
	wid.tips.set_tip(b,_('Click to grab a new keyboard shortcut'))
	wid.tips.set_tip(clr,_('Click to clear the keyboard shortcut'))
	clr.connect('clicked',clr_clicked)
	box.pack_start(l, True, True)
	box.pack_end(clr, False, False)
	box.pack_end(b, False, False)

	return [box]
	

def build_mytoggle(box, node, label, option):
	toggle = g.CheckButton(label)
	box.may_add_tip(toggle, node)
	def get():
		if toggle.get_active():
			return 'true'
		else:
			return 'false'
	def set():
		if option.value == 'true':
			toggle.set_active(True)
		else:
			toggle.set_active(False)
		
	box.handlers[option] = (get, set)
	toggle.connect('toggled', lambda w: box.check_widget(option))
	return [toggle]

def build_buttonfilter(box, node, label, option):
	vbox = g.VBox()
	l=g.Label('')
	l.set_markup('<b>'+label+'</b>')
	l.set_padding(0,4)
	vbox.add(l)
	vbox.set_border_width(8)
	
	toggles = {}
	for (char, label) in [
		('T','Sticky'),
		('S','Shade'),
		('H','Hide'),
		('M','Maximize'),
		('C','Close')
		]:
		t = g.CheckButton(label)
		toggles[char]=t
		vbox.add(t)
		t.connect('toggled', lambda w: box.check_widget(option))
#		box.tips.set_tip(t,
#'Enable / disable the %s button on the titlebar.' % (label))

	def get():
		s = ''
		for (char,t) in toggles.iteritems():
			if not t.get_active(): s += char
		return s
		
	def set():
		for t in toggles.values(): t.set_active(True)
		for char in option.value: toggles[char].set_active(False)
		
	box.handlers[option] = (get, set)
	return [vbox]

OptionsBox.widget_registry['keybutt'] = build_keybutt
OptionsBox.widget_registry['mytoggle'] = build_mytoggle
OptionsBox.widget_registry['buttonfilter'] = build_buttonfilter

rox.app_options = OptionGroupRC(app_name,'workspacerc')

Opt('workspace_count')
Opt('cycle_workspaces')
Opt('wrap_workspaces')

#Opt('next_workspace_key')
#Opt('prev_workspace_key')
#Opt('add_workspace_key')
#Opt('del_workspace_key')
#Opt('workspace_1_key')
#Opt('workspace_2_key')
#Opt('workspace_3_key')
#Opt('workspace_4_key')
#Opt('workspace_5_key')
#Opt('workspace_6_key')
#Opt('workspace_7_key')
#Opt('workspace_8_key')
#Opt('workspace_9_key')
#Opt('move_window_next_workspace_key')
#Opt('move_window_prev_workspace_key')
#Opt('move_window_workspace_1_key')
#Opt('move_window_workspace_2_key')
#Opt('move_window_workspace_3_key')
#Opt('move_window_workspace_4_key')
#Opt('move_window_workspace_5_key')
#Opt('move_window_workspace_6_key')
#Opt('move_window_workspace_7_key')
#Opt('move_window_workspace_8_key')
#Opt('move_window_workspace_9_key')

rox.app_options.add_notify(update_options)
rox.app_options.notify()
rox.edit_options()
